#! /usr/bin/env ruby

# ncal.rb: Written by Tadayoshi Funaba 1998-2002
# $Id: ncal.rb,v 2.2 2002-03-19 22:16:09+09 tadf Exp $

require 'date2'
require 'date2/holiday'
require 'getopts'

$tab =
{
  'cn' => true,    # China
  'de' => 2342032, # Germany (protestant states)
  'dk' => 2342032, # Denmark
  'es' => 2299161, # Spain
  'fi' => 2361390, # Finland
  'fr' => 2299227, # France
  'gb' => 2361222, # United Kingdom
  'gr' => 2423868, # Greece
  'hu' => 2301004, # Hungary
  'it' => 2299161, # Italy
  'jp' => true,    # Japan
  'no' => 2342032, # Norway
  'pl' => 2299161, # Poland
  'pt' => 2299161, # Portugal
  'ru' => 2421639, # Russia
  'se' => 2361390, # Sweden
  'us' => 2361222, # United States
  'os' => false,   # (old style)
  'ns' => true     # (new style)
}

$cc = 'jp'

def usage
  $stderr.puts 'usage: ncal [-c iso3166] [-jmty] [[month] year]'
  exit 1
end

def pict(y, m, sg)
  d = (1..31).detect{|d| Date.exist?(y, m, d, sg)}
  fi = Date.new(y, m, d, sg)
  fi -= (fi.jd - $k + 1) % 7

  ve  = (fi..fi +  6).collect{|cu|
    [ %w(      )[cu.wday],
      [31, 0, 0, 0, 0, 0, 34][cu.wday] ]
  }
  ve += (fi..fi + 41).collect{|cu|
    [ if cu.mon == m then cu.send($da) end.to_s,
      if cu.mon == m and cu.nhol? then 41
      else [31, 0, 0, 0, 0, 0, 34][cu.wday] end ]
  }

  ve = ve.collect{|e, n| esc(e.rjust($dw), n)}

  gr = group(ve, 7)
  gr = trans(gr) if $OPT_t
  ta = gr.collect{|xs| xs.join(' ')}

  ca = %w(ӷ ǡ    ̵
	  ʸ շ Ĺ ̵  )[m - 1]
  ca = y.to_s + 'ǯ ' + ca if not $OPT_y
  ca = esc(ca.center($mw), 1)

  ta.unshift(ca)
end

def seq(n)
  if $es then "\e[" + n.to_s + "m" else '' end
end

def esc(s, n)
  seq(n) + s + seq(0)
end

def group(xs, n)
  (0..xs.size / n - 1).collect{|i| xs[i * n, n]}
end

def trans(xs)
  (0..xs[0].size - 1).collect{|i| xs.collect{|x| x[i]}}
end

def unite(xs)
  if xs.empty? then [] else xs[0] + unite(xs[1..-1]) end
end

def block(xs, n)
  unite(group(xs, n).collect{|ys| trans(ys).collect{|zs| zs.join('  ')}})
end

def unlines(xs)
  xs.collect{|x| x + "\n"}.join
end

usage unless getopts('jmty', "c:#{$cc}")

y, m = ARGV.indexes(1, 0).compact.collect{|x| x.to_i}
$OPT_y ||= (y and not m)

to = Date.today
y ||= to.year
m ||= to.mon

usage unless m >= 1 and m <= 12
usage unless y >= -4712
usage unless sg = $tab[$OPT_c]

$dw = if $OPT_j then 3 else 2 end
$mw = ($dw + 1) * 7 - 1
$mn = if $OPT_j then 2 else 3 end
$tw = ($mw + 2) * $mn - 2

$k  = if $OPT_m then 1 else 0 end
$da = if $OPT_j then :yday else :mday end
$es = $stdout.tty?

print (if not $OPT_y
	 unlines(pict(y, m, sg))
       else
	 esc((y.to_s + 'ǯ').center($tw), 1) + "\n\n" +
	   unlines(block((1..12).collect{|m| pict(y, m, sg)}, $mn)) + "\n"
       end)
